local T = ...
local spickle_func = spickle2

T.BGColors = {
	[0] = string.format("%s %d", ListColors[0], ListColors.Alpha),
	[1] = string.format("%s %d", ListColors[1], ListColors.Alpha),
	[2] = string.format("%s %d", ListColors[2], ListColors.SelectedAlpha),
	[3] = string.format("%s %d", "255 255 255", 255),
	[4] = string.format("%s %d", "255 50 50", 255),
	Numbers = {
		[0] = ListColors.Numbers[0],
		[1] = ListColors.Numbers[1],
		[2] = ListColors.Numbers[2],
		[3] = {255, 255, 255, 255},
		[4] = {255, 50, 50, 255},
	
	},
}

T.RoidNames = {
	["Ice Crystal"] = true,
	["Asteroid"] = true,
}

T.ShipsWithMineralScanners = {
	["Warthog Mineral Extractor"] = 500,
	["Behemoth Heaver Miner MkII"] = 1000,
	["Goliath"] = 500,
	["Trident Light Frigate"] = 500,
}

local ore_reference = {
	[1] = {
		'Aq',
		'Si',
		'Ca',
		'Fe',
		'Is',
		'Va',
		'Xi',
		'La',
		'De',
		'Py',
		'Ap',
		'Pe',
		'He',
		'Sa',
	},
    [2] = {
		Aq = 'Aquean',
		Si = 'Silicate',
		Ca = 'Carbonic',
		Fe = 'Ferric',
		Is = 'Ishik',
		Va = 'VanAzek',
		Xi = 'Xithricite',
		La = 'Lanthanic',
		De = 'Denic',
		Py = 'Pyronic',
		Ap = 'Apicene',
		Pe = 'Pentric',
		He = 'Heliocene',
		Sa = 'Sammic',
	},
    [3] = {
		Aquean = 'Aq',
		Silicate = 'Si',
		Carbonic = 'Ca',
		Ferric = 'Fe',
		Ishik = 'Is',
		VanAzek = 'Va',
		Xithricite = 'Xi',
		Lanthanic = 'La',
		Denic = 'De',
		Pyronic = 'Py',
		Apicene = 'Ap',
		Pentric = 'Pe',
		Heliocene = 'He',
		Sammic = 'Sa',
	}
}
for i, v in ipairs(ore_reference[1]) do ore_reference[1][v] = i end
T.ore_reference = ore_reference


-- Holes all of the roid data.  use T.GetData() to access
local asteroid_data

-- Holds calculated summaries for systems and sectors
local summary_data

-- Holds sectors needing to be updated because of newly scanned roids.
-- Updates should only be processed when viewing the UI, not during active flight
local new_data = false -- flag to track new data for saving
do
	local count = 0
	T.pending = {
	--	[<sectorid>] = {
	-- 		[<roidID>] = true,
	-- 	},
		AddRoid = function(self, sectorid, roidID, roidtbl)
			self[sectorid] = self[sectorid] or {}
			self[sectorid][roidID] = roidtbl
			count = count + 1
			new_data = true
			if (not T.Save.timer:IsActive()) then
				T.Save.timer:SetTimeout(T.Save.interval, function() T.SaveData(GetSystemID(sectorid)) end)
			end
		end,

		Clear = function(self, sectorid)
			if (sectorid) then
				self[sectorid] = nil
				count = count - 1
			end
		end,

		GetCount = function()
			return count
		end,

		Update = function(self)
		end
	}
end

local function process_roidtbls(system_summary, sector_summary, roidtbl)
	for ore_type, ore_pct in pairs(roidtbl.ore) do

		-- System Summary
		system_summary[ore_type] = system_summary[ore_type] or {highest = 0, count = 0}
		if (ore_pct > (system_summary[ore_type].highest or 0)) then
			system_summary[ore_type].highest = ore_pct
		end
		system_summary[ore_type].count = system_summary[ore_type].count + 1

		-- Sector Summary
		sector_summary[ore_type] = sector_summary[ore_type] or {highest = 0, count = 0}
		if (ore_pct > (sector_summary[ore_type].highest or 0)) then
			sector_summary[ore_type].highest = ore_pct
		end
		sector_summary[ore_type].count = sector_summary[ore_type].count + 1
	end
end


local function populate_sector_summary(sectorid, refresh)
	-- Populate Sector Summary and updaate system summary
	-- Both operations require iterating over all of the roids
	local systemid = GetSystemID(sectorid)

	local sector_data = T.GetData(systemid, sectorid) -- not using create_table because it should only be created by scanning

	if (not next(sector_data)) then -- don't process an empty sector.  keep the dataset clean
		return {}
	end

	-- Establish summary data structure
	summary_data[systemid] = summary_data[systemid] or {}
	local system_summary = summary_data[systemid]
	system_summary[sectorid] = system_summary[sectorid] or {}
	local sector_summary = system_summary[sectorid]

	if (next(sector_summary)) then
		-- summary exists
		if (T.pending[sectorid]) then
			-- process pending updates
			for _,  roidtbl in pairs(T.pending[sectorid]) do
				process_roidtbls(system_summary, sector_summary, roidtbl)
			end
		end
	else
		-- no summary yet
		for roidid, roidtbl in pairs(sector_data) do
			process_roidtbls(system_summary, sector_summary, roidtbl)
		end
	end
	T.pending:Clear(sectorid) -- roids will be pending even if no summary exists.  clear pending after either operation

	return sector_summary
end

local function populate_system_summary(systemid, refresh)
-- 	if (summary_data[systemid] and not refresh) then
-- 		return summary_data[systemid]
-- 	end
	local system_data = T.GetData(systemid)


	if (not next(system_data)) then -- if no system data, there's nothing to do.  get out of here.
		return {}
	end

	summary_data[systemid] = summary_data[systemid] or {}
	
	-- Cycle through all sectors and roids to find the highest percent and roid count, by ore type
	for sectorid, sectortbl in pairs(system_data) do
		if (type(sectorid) == "number" and next(sectortbl)) then
			-- next() is to avoid possible empty sectors.  Might just be a problem with my data set from testing.
			-- System summary is updated at the same time as sector summary, since they both have to iterate over all of the roids
			-- Summaries should be updated when roids are scanned.  Only check for sectors not yet in the system summary
			populate_sector_summary(sectorid)
		end
	end
	return summary_data[systemid]
end

local function populate_universe_summary()
	for systemid, systemtbl in pairs(T.GetData()) do
		populate_system_summary(systemid)
	end
	return summary_data or {}
end

-- TODO probably should move the populate series of functions into these functions below
function T.GetSectorSummary(sectorid)
	if (not sectorid) then return {} end
	local systemid = GetSystemID(sectorid)
	return populate_sector_summary(sectorid)
end

function T.GetSystemSummary(systemid)
	if (not systemid) then return {} end
	return populate_system_summary(systemid)
end

function T.GetUniverseSummary()
	-- verify all systems have been parsed
	populate_universe_summary()
	return summary_data or {}
end

function T.GetData(systemid, sectorid, create_table, delete_table)
	local data
	if (not systemid) then 
		data = asteroid_data
	else
		asteroid_data[systemid] = asteroid_data[systemid] or create_table and {}
		data = asteroid_data[systemid]
		
		if (data and sectorid) then
			if (delete_table) then
				data[sectorid] = nil
			else
				data[sectorid] = data[sectorid] or create_table and {}
			end
			data = data[sectorid]
		end
	end
	return data or {}
end

-- Outside-facing wrapper for T.GetData() to allow other plugins to... get data.
function roiddb.GetAsteroidData(systemID, sectorID, roidID)
	if (not (systemID or (systemID and sectorID))) then return end

	local data = T.GetData(systemID, sectorID)
	if (systemID and sectorID and roidID) then
		data = data[roidID]
	end
	return data
end

function T.DeleteSectorData(sectorid)
	T.GetData(GetCurrentSystemid(), GetCurrentSectorid(), nil, true)
	T.SaveData(GetCurrentSystemid(), true)
end

function T.LoadData(systemid)
	if (new_data) then T.SaveData() end

	-- Local data
	local local_data_string = LoadSystemNotes(roiddb.notesid)
	local_data_string = local_data_string ~= "" and local_data_string or "pinned = {}"
	T.LocalData = unspickle(local_data_string)

	summary_data = {}
	--	asteroid_data = dofile("plugins/mdi-roiddb/roids.lua")
	asteroid_data = {} -- ?always initialize with empty table?
	for systemid = 1, T.HighestSystemID do
		local notes_data = LoadSystemNotes(roiddb.notesid + systemid)
		if (notes_data ~= "") then -- don't dirty the dataset if unnecesssary
			asteroid_data[systemid] = unspickle(notes_data)
		end
	end
	-- If asteroid dataset contains unknown system data, then modifiy the SystemNames to make references possible
	if (asteroid_data[33]) then
		SystemNames[33] = "Unknown"
		SystemNames["unknown"] = 33
		T.UI.unknownbtn.visible = "YES"
	end
end

function T.SaveData(systemid, force)
	if (new_data or force) then
		T.Save.timer:Kill()
		new_data = false
		local data

		if (systemid) then
			data = {[systemid] = T.GetData(systemid)}
		else
			data = T.GetData()
		end

		for systemid, system_data in pairs(data) do
			local notes = spickle_func(system_data)
			if (notes ~= "") then
				SaveSystemNotes(notes, roiddb.notesid + systemid)
			end
		end
	end
end

T.Export = {}
function T.Export.SuccessMsg(source, to)
	T.MessageDlg:SetMessage(source.." data has been exported "..to..".", "OK", function(self) HideDialog(iup.GetDialog(self)) end)
	ShowDialog(T.MessageDlg)
end

function T.Export.RoidDB(notesid)
	SaveSystemNotes(spickle_func(T.GetData()), notesid or -roiddb.notesid)
	T.Export.SuccessMsg("RoidDB", "to \"system"..-roiddb.notesid.."notes.txt\"")
end

T.Import = {}
function T.Import.NoDataMsg(source)
	T.MessageDlg:SetMessage("There is no "..source.." data to import.", "OK", function(self) HideDialog(iup.GetDialog(self)) end)
	ShowDialog(T.MessageDlg)
end

function T.Import.SuccessMsg(source, count)
	T.MessageDlg:SetMessage(count.." asteroids imported from "..source..".", "OK", function(self) HideDialog(iup.GetDialog(self)) end)
	ShowDialog(T.MessageDlg)
end


function T.Import.RoidDB(notesid)
	local import_data = unspickle(LoadSystemNotes(notesid or -roiddb.notesid))
	local count = 0

	if (not next(import_data)) then
		T.Import.NoDataMsg("RoidDB")
		return
	end

	for systemID, system_data in pairs(import_data) do
		for sectorID, sector_data in pairs(system_data) do
			local data = T.GetData(systemID, sectorID, true)

			for roidID, roidtbl in pairs(sector_data) do
				if (not data[roidID] or T.Scan.re) then
					data[roidID] = roidtbl
					count = count + 1
				end
			end
		end
	end

	--asteroid_data = data
	T.Import.SuccessMsg("RoidDB", count)
	new_data = true
	T.SaveData()
end

function T.Import.Prospector()
	local PR_SystemNotesPrefix = 91000000 -- hard-coded in case prospector isn't installed
	local count = 0
	local gotsome = false
	-- Prospector saves each sector in it's own SystemNotes file.  This seems highly irresponsible to me.
	for sectorid = 1, ((T.HighestSystemID - 1) * 256 + (16 - 1) * 16 + 16) do -- sectorid = (s-1)*256 + (x-1)*16 + y
		local notes_data = LoadSystemNotes(PR_SystemNotesPrefix + sectorid)
		if (notes_data ~= "") then
			local data = T.GetData(GetSystemID(sectorid), sectorid, true)
			notes_data = unspickle(notes_data)
			gotsome = true
			for roidID, roidtbl in pairs(notes_data.RoidList) do
				if (not data[roidID] or T.Scan.re) then
					count = count + 1
					data[roidID] = {id = roidID, ore = {}}
					for ore_type, ore_pct in pairs(roidtbl) do
						data[roidID].ore[ore_type] = ore_pct
					end
				end
			end

		end
	end
	if (not gotsome) then
		T.Import.NoDataMsg("Prospector")
		return
	end
	T.Import.SuccessMsg("Prospector", count)
	new_data = true
	T.SaveData()
end

function T.Import.TargetLess()
	-- TargetLess data is stored by sector, with sector data being spickled separately from the universe
	local tldata = unspickle(LoadSystemNotes(741937)) -- unspickle the universe
	local count = 0
	if (not next(tldata)) then
		T.Import.NoDataMsg("TargetLess")
		return
	end

	for sectorid, sector_data in pairs(tldata) do
		sector_data = unspickle(sector_data)
		if (next(sector_data)) then
			local data = T.GetData(GetSystemID(sectorid), sectorid, true) -- RoidDB data is stored with sector data under the representative systemid
			for roidID, roidtbl in pairs(sector_data) do -- unspickle the sector data and operate over the roid data
				if (not data[roidID] or T.Scan.re) then -- only update roid data if it's a new roid
					count = count + 1
					data[roidID] = {id = roidID, ore = {}}
					for ore_type, ore_pct in pairs(roidtbl.ore) do
						data[roidID].ore[ore_reference[3][ore_type]] = tonumber(ore_pct)
					end
				end
			end
		end
	end
	T.Import.SuccessMsg("TargetLess", count)
	new_data = true
	T.SaveData()
end


function T.Import.OldRoidDB()
	local data = unspickle(LoadSystemNotes(764332))
	local count = 0

	if (not next(data)) then
		T.Import.NoDataMsg("Old RoidDB")
		return
	end

	for sectorid, sector_data in pairs(data) do
		local data = T.GetData(GetSystemID(sectorid), sectorid, true) -- RoidDB data is stored with sector data under the representative systemid
		for roidID, roidtbl in pairs(sector_data) do

			if (not data[roidID] or T.Scan.re) then
				count = count + 1
				data[roidID] = {id = roidID, ore = {}}
				for ore_type, ore_pct in pairs(roidtbl) do
					data[roidID].ore[ore_reference[3][ore_type]] = ore_pct
				end
			end

		end
	end
	T.Import.SuccessMsg("Old RoidDB", count)
	new_data = true
	T.SaveData()
end


T.help_text = [=[

MDI-RoidDB commands:
/roiddb
/roiddbscan
/roiddbdeletesectordata

Filters
-----------------------------------
The filter options on the right can be used to restrict the results from a search, show only sectors with that ore on the system view, or show only asteroids with that ore in the sector view.  You can further filter the display by specifying a minimum percent of ore you are looking for.

Search
-----------------------------------
Search will return a list of systems or sectors with scanned asteroids that match the specified filter.


Universe View
-----------------------------------
From the universe view, select a system to view the sectors you have asteroids scanned in.  If you have scanned any asteroids in the Unknown system, there will be an additional button to select this system for viewing.

System View
-----------------------------------
The system view displays all of the sectors in the system that you have scanned asteroids in.  The highlighted systems will change to respect the filter you have selected.

Sector View
-----------------------------------
The sector view displays all of the asteroids you have scanned in the sector.  You can use the filters to only display asteroids with the specified ore.  The columns can be sorted by clicking on the headers at the top of the display.  The check boxes on the left side of the display can be used to pin the asteroid to the top of the display.  When an asteroid is selected, you can double-click it or press the target button to set your current target to that asteroid, assuming it is within range.  This action will close the display.

Auto-Scanner (/roiddbscan)
-----------------------------------
The auto-scanner will attempt to find an asteroid, within scanner range, that has not yet been scanned.  It will disengage automatically if there are no unscanned asteroids in radar range or if a non-asteroid is manually targeted.

Other
-----------------------------------
If you take damage while the interface is open, it will automatically close so that you can respond to the source of damage.

MDI-TargetList
-----------------------------------
If you have MDI-TargetList v4+ installed, you will be able to view a list of asteroids that have been scanned, or that are unscanned, from the HUD.
]=]

-- -- -------------------------------
-- sectorid = (s-1)*256 + (x-1)*16 + y
-- -- -------------------------------


-- Roid Data
-- roid_data_table = {
-- 	[<system_id>] = {
-- 		[<sector_id>] = {
-- 			[<roid_id>] = {
-- 				id = <number>
-- 				ore = {
-- 					[<ore_type>] = <number>, -- <ore_type> should be 2-letter notation
-- 				},
-- 			},
-- 		},
-- 	}
-- }

-- Summary Data
-- 	[<systemid>] = {
-- 		[<sectorid>] = {
-- 			-- Sector ore summary
-- 			[[ore_type]] = {
-- 				count = <number>,
-- 				highest = <number>, -- (0 - 100)
-- 			},
-- 		},
-- 		-- System ore summary
-- 		[[ore_type]] = {
-- 			count = <number>,
-- 			highest = <number>, -- (0 - 100)
-- 		},
-- 	},
