local stack, new_spickle, process_table, table_sort, escape_text
local sformat = string.format
local srep = string.rep
local tinsert = table.insert
local tremove = table.remove
local tsort = table.sort

local function xor(a, b) return ((a and not b) or (not a and b)) end

local function escape_text(str)
	str = str:gsub("[\"\'\\\n]", "\\%1")
	return str
end

local function CreateStack()
	stack = {}

	function stack:push(ref) tinsert(self, ref) end
	function stack:pop() return tremove(self) end
	function stack:onstack(ref)
		local retval = false
		for _, v in ipairs(self) do
			if (v == ref) then
				retval = true
				break
			end
		end
		return retval
	end
	function stack:getstackpath(ref)
		if (not self:onstack(ref)) then return "Not on stack" end

		local path = '<initial table>'
		local step = ''
		if (self[1] ~= ref) then
			for i = 2, #self do
				for k, v in pairs(self[i - 1]) do
					if (v == self[i]) then
						local tk = type(k)
						if     (tk ~= 'string') then
							step = sformat('[%s]', tostring(k))
						else
							step = sformat('.%s', k)
						end
						path = path..step
						break
					end
				end
				if (self[i] == ref) then break end
			end
		end
		return path
	end
	return stack
end

-- ### ######################################################################### ###
-- ### ######################################################################### ###
-- ### ######################################################################### ###
-- ### ######################################################################### ###
-- ### ######################################################################### ###

table_sort = function(a, b)
	local retval
	local oa, ob = a, b
	a, b = tonumber(a), tonumber(b)
	-- sort numbers before anything else
	if (a and b) then
		retval = a < b
	-- if one of the parameters is not a number, then sort the number first
	elseif (xor(a, b)) then
		retval = a -- if a exists, then a < b

	-- sort everything else
	else
		local ais, bis = type(oa) == "string", type(ob) == "string"
		a, b = tostring(oa):lower(), tostring(ob):lower()

		-- sort strings before other non-number datatypes
		if (ais and bis) then
			retval = a < b

		-- if one of the parameters is not a string, then sort the string first
		elseif (xor(ais, bis)) then
			retval = ais

		-- finally sort everything else
		else
			retval = a < b
		end
	end
	return retval
end

-- ### ######################################################################### ###
-- ### ######################################################################### ###
-- ### ######################################################################### ###
-- ### ######################################################################### ###
-- ### ######################################################################### ###

process_table = function(tbl, count)
	local msg = ''
	local indent = '\n'..srep('\t', count-1)
	local idx = ''
	local n = 0

	assert(not stack:onstack(tbl), 'trying to spickle a recursive table structure (c)! no way, Jim Bob.')
	stack:push(tbl)

	local sorted_table = {}
	for k, v in pairs(tbl) do
		tinsert(sorted_table, k)
	end
	tsort(sorted_table, table_sort)
	for i, v in ipairs(tbl) do
		n = i
		msg = msg..indent..new_spickle(v, count)..","
	end

	for _, k in ipairs(sorted_table) do
		local tk = type(k)
		if (not (tk == 'number' and k > 0 and k <= n)) then
			local v = new_spickle(tbl[k], count)
			if (tk == 'number') then
				k = '['..k..']'
			elseif (tk == 'string') then
				k = '["'..k..'"]'
			end
			msg = msg..indent..tostring(k).." = "..v..","
		end
	end
	stack:pop()
	return msg
end

-- ### ######################################################################### ###
-- ### ######################################################################### ###
-- ### ######################################################################### ###
-- ### ######################################################################### ###
-- ### ######################################################################### ###

new_spickle = function(v, count)

	local t = type(v)
	if (t == 'table') then
		count = (count or 0) + 1
		local pre = count > 1 and '{' or ''
		local post = count > 1 and '\n'..srep('\t', count-2)..'}' or ''
		v = pre..process_table(v, count)..post
		count = count - 1
	elseif (t == 'string') then
		v = '"'..escape_text(v)..'"'
	else
		v = tostring(v)
	end
	return v
end

-- ### ######################################################################### ###
-- ### ######################################################################### ###
-- ### ######################################################################### ###
-- ### ######################################################################### ###
-- ### ######################################################################### ###

stack = CreateStack()

local function receiver(data)
	return new_spickle(data)
end

declare("spickle2", receiver)
