local T, loadluafile, check_number = ...

local ore_reference = T.ore_reference

local ui_page
local start_page = 3

loadluafile("ui_navmap.lua", T)
loadluafile("ui_matrix.lua", T)
loadluafile("ui_options.lua", T, check_number)


local BuildUI, BuildSearchUI, BuildStatus, BuildHamburgerUI

-- Get the width of a control based a given title string and font
-- generator: <string> iup function name to use (e.g. label, toggle)
-- str:       <string> string to set as the title
-- font:      <number> font size
local function get_control_width(generator, str, font)
	local c = iup[generator]{title = str, value = str, font = font, expand = "NO"}
	iup.Map(c)
	local w = tonumber(c.w)
	local h = tonumber(c.h)
	iup.Destroy(c)
	return w, h
end

-- Check for the presence of a given key/filter within a table
-- data:   <table> table containing the specified filter
-- filter: <string> key to search for
local function check_filters(data, filter)
	if (filter == "All" or filter == nil) then
		return true
	elseif (data[filter]) then 
		return true 
	end
	return false
end

local function num_text_action(Lval, Uval, dp, action)
	-- Action function that will only allow number-based entries within a given range
	-- NOTE:  Doesn't support negative values
	-- <Lval>, <Uval>:  Lower and Upper bounds in the number range
	-- [dp]: how many decimal places to consider, or the [action] function
	-- [action]:  function to be called upon success
	if (type(dp) == "function") then
		action = dp
		dp = nil
	end

	return function(self, c, after)
		local num = tonumber(after)
		if (c == 8  or c == iup.K_BS or -- backspace
			c == 9  or -- tab
			c == 27 or -- escape
			c == 13 or -- enter
			dp and (c == 46 or c == 48) and check_decimal_prefix(after, dp) or -- 46: .   48: 0
			check_number(num, Lval, Uval, dp)
		) then
			if (not dp and c == 46) then return iup.IGNORE end -- if there's no decimal allowed and they try to enter a decimal point
			if (type(action) == "function") then action(c, num) end
			--make_dirty(is_mobile_input_good)
			is_mobile_input_good()
			return iup.DEFAULT
		else
			return iup.IGNORE
		end
	end
end

	local function mobile_check_control(ctrl, l, h, d)
		if (check_number(ctrl.value, l, h, d) or ctrl.value == "") then
			ctrl.fgcolor = "255 255 255"
		else
			ctrl.fgcolor = "255 0 0"
			return false
		end
		return true
	end

	function is_mobile_input_good()
		local ok_to_apply = true
		ok_to_apply = mobile_check_control(T.UI.search_min, 0, 100) and ok_to_apply
		return ok_to_apply
	end



-- Calcuate a sector ID baed on the short form sector notation
-- loc:  <string> location string (e.g. "Setalli Shinas D-10"))
local function GetSectorIDFromShortStr(l)
	local s, y, x = l:match("^(%w+)[%w ]+(%w)-(%d+)")
	s = SystemNames[s:lower()]
	y = y:byte() - iup.K_A + 1
	return (s - 1) * 256 + (x - 1) * 16 + y 
end

-- Get the system id from a system name
-- n:  <string> system name (e.g. "Setalli Shinas")
local function GetSystemIDFromName(n)
	n = n:match("^(%w+)[%w ]+")
	return SystemNames[n:lower()]
end

-- ### ######################################################### ###
-- ### ######################################################### ###
-- ###                                                           ###
-- ###                         Main UI                           ###
-- ###                                                           ###
-- ### ######################################################### ###
-- ### ######################################################### ###
BuildUI = function(layout)
	local matrix_container, matrix, matrix_location
	local navmap, navmap_container, unknownbtn
	local prevbtn, nextbtn, setpathbtn, clearpathbtn, closebtn, optionsbtn, cursectorbtn, targetbtn, hamburger
	local searchbtn, search_min
	local scanned_total, est_max_total
	local filters_and_highest
	local desc_header
	local filters, filter_radio, filter_display
	local reloadbtn
	local main_display_zbox
	local left_container, lower_left_container
	local right_container, lower_right_container
	local button_container_content
	local dlg, container

-- ### ######################################################### ###
-- ### Main UI - Interface Layout                                ###
-- ### ######################################################### ###
	local button_width = get_control_width("label", string.rep('X', string.len("< Universe Map")), T.Font)
	prevbtn       = iup.stationbutton{title = "Previous", size = button_width, active = "NO", font = T.Font, hotkey = iup.K_less}
	nextbtn       = iup.stationbutton{title = "Next", size = button_width,  active = "NO", font = T.Font, hotkey = iup.K_greater}
	cursectorbtn  = iup.stationbutton{title = "Current Sector", size = button_width, font = T.Font}
	setpathbtn    = iup.stationbutton{title = "Set Path", size = button_width, active = "NO", font = T.Font}
	clearpathbtn  = iup.stationbutton{title = "Clear Path", size = button_width, active = "NO", font = T.Font}
	searchbtn     = iup.stationbutton{title = "Search", size = button_width, active = "YES", font = T.Font}
	search_min    = iup.text{value = "0", size = get_control_width("label", "0000"), active = "NO", font = T.Font}
	scanned_total = iup.label{title = "0", font = T.Font, alignment = "ARIGHT", expand = "YES"}
	est_max_total = iup.label{title = "0", font = T.Font, alignment = "ARIGHT", expand = "YES"}

	desc_header   = iup.label{title = "", expand = "HORIZONTAL", font = Font.H1 * T.FONT_SCALE}

	optionsbtn    = iup.stationbutton{title = "Options", size = button_width, font = T.Font}
	closebtn      = iup.stationbutton{title = "Close",  size = button_width, font = T.Font, visible = T.IsMobile() and "NO" or "YES"}
	reloadbtn     = iup.stationbutton{title = "Reload", font = T.Font, hotkey = iup.K_r}
	targetbtn     = iup.stationbutton{title = "Target", font = T.Font}

	hamburger     = iup.stationbutton{title = "=", font = T.Font, visible = T.IsMobile() and "YES" or "NO"}
	
	local ore_width = get_control_width("label", "000.0", T.Font)

	filters_and_highest = {
		iup.hbox{
			iup.stationradio{title = "All", value = "ON", font = T.Font,},
			iup.fill{},
			iup.label{title = "", font = T.Font,},
			gap = 25,
		},
		expand = "NO",
	}
	for i, ore_type in ipairs(ore_reference[1]) do
		table.insert(filters_and_highest, iup.hbox{
			iup.stationradio{title = ore_reference[2][ore_type], font = T.Font,},
			iup.fill{},
			iup.label{title = "-", size = ore_width, font = T.Font},
			gap = 25,
		})
	end

	filter_radio = iup.radio{iup.vbox(filters_and_highest)}
	filter_radio.value = filters_and_highest[1][1]

	filter_display = iup.vbox{
		iup.label{title = "Filter / Highest %", font = T.Font},
		filter_radio,
		alignment = "ACENTER",
	}

	matrix_container, matrix = T:BuildMatrix(T.MatrixLayout["sector_roid_list"], {expand = "YES"})
	navmap_container, navmap, unknownbtn = T:BuildNavmap()

	main_display_zbox = iup.zbox{
		navmap_container,
		matrix_container,
	}
	
	local frame_gap = 10
	local button_gap = 10
	local control_gap = 20


	lower_left_container = iup.hbox{
		--reloadbtn,
		targetbtn,
	}
	if (T.IsMobile()) then
		iup.Append(lower_left_container, closebtn)
	end

	left_container = iup.vbox{ -- left-side, vertical display
		iup.pdarootframe{
			iup.hbox{
				desc_header,
				hamburger,
			},
		},
		main_display_zbox,
		iup.pdarootframe{ -- button bar at the bottom of the screen
			iup.hbox{
				lower_left_container,
				iup.fill{},
				prevbtn,
				nextbtn,
				-- ------------------
				gap = button_gap,
			}, -- hbox
		}, -- pdarootframe
		gap = frame_gap,
	} -- vbox




	button_container_content = {
		setpathbtn,
		clearpathbtn,
		iup.fill{},
		cursectorbtn,
		iup.fill{},
		optionsbtn,
		closebtn,
		-- --------------
		gap = button_gap,
		alignment = "ACENTER",
		margin = "5x",
	}

	lower_right_container = iup.pdarootframe{ -- miscellaneous stuff
		iup.hbox{
			iup.fill{},
			iup.vbox{
				iup.vbox{
					iup.hbox{iup.fill{size = button_width}},
					iup.hbox{
						iup.label{title = "Scanned:", font = T.Font},
						iup.fill{},
						scanned_total,
					}, -- hbox
					iup.hbox{
						iup.label{title = "Est. Total:", font = T.Font},
						iup.fill{},
						est_max_total,
					}, -- hbox
					-- --------------
					expand = "NO",
					--margin = "x20",
				}, -- vbox
				iup.fill{},
				iup.vbox(T.IsMobile() and {} or button_container_content),
			}, -- vbox
			iup.fill{},
		}, -- hbox
	} -- pdarootframe -- miscellaneous stuff

	right_container = iup.vbox{
		iup.pdarootframe{ -- filter control group
			iup.vbox{
				filter_display,
				searchbtn,
				iup.hbox{
					iup.label{title = "Min", alignment = "ARIGHT"},
					search_min,
					iup.label{title = "%"},
					-- --------------
					gap = 5,
					alignment = "ACENTER",
				}, -- hbox
				-- --------------
				gap = 20,
				margin = "5x20",
				alignment = "ACENTER",
			}, -- vbox
		}, -- pdarootframe
		lower_right_container,
		--iup.fill{size = 50},
		-- --------------
		gap = frame_gap,
		alignment = "ACENTER",
	} -- vbox

	container = iup.hbox{
		iup.vbox{
			iup.hbox{
				left_container,
				right_container,
				-- --------------
				alignment = "ACENTER",
				gap = frame_gap,
			}, -- hbox
			gap = frame_gap,
		}, -- vbox
		margin = "25x25",
	} -- hbox

	dlg = iup.dialog{
		container,

		topmost = "YES",
		fullscreen = "YES",
		defaultesc = closebtn,
		bgcolor = "0 0 0 128 *"
	}
    
-- ### ######################################################### ###
-- ### Main UI - Interface Callbacks & Supporting Code           ###
-- ### ######################################################### ###
	local change_page
	ui_page = nil
	-- Filter action callbacks
	for i, c in ipairs(filters_and_highest) do
		c[1].action = function(self, v)
			if (v == 1) then
				filter_radio.value = self
				searchbtn.active = ui_page < 3 and "YES" or "NO" -- and (self.title == "All" and "NO" or "YES")
				search_min:ToggleActive()
				if     (ui_page == 1) then -- navmap: Universe
					
				elseif (ui_page == 2) then -- navmap: System
					navmap:refresh(ore_reference[3][self.title])-- or "All") -- show only sectors with ore of the filter type

				elseif (ui_page == 3) then -- matrix: Sector
					matrix:refresh(nil, ore_reference[3][self.title] or "All") -- limit the matrix display to only roids containing the filter
				end
			end
		end
	end

	-- Sets the Highest Ore Displays according to the generated summaries for the given system or sector
	function filters_and_highest:SetHighest(systemid, sectorid)
		local highest = {}
		local summary

		-- display the highest ore % in the system
		if (ui_page == 1 or (ui_page == 2 and not sectorid)) then
			summary = T.GetSystemSummary(systemid)

		-- display the highest ore % in the sector
		elseif (ui_page > 1) then
			summary = T.GetSectorSummary(sectorid)
		end
		for i, ore_type in ipairs(ore_reference[1]) do
			local ore_pct = summary[ore_type] and summary[ore_type].highest > 0 and summary[ore_type].highest or "-"
			self[i + 1][3].title = ore_pct -- index 1 is "All" and receives no ore percent
		end

		
	end
	
	search_min.action = num_text_action(0, 100, function() T.UI.navmap:refresh(ore_reference[3][filter_radio.value.title]) end)
	-- Make the search_min text box (in)active and color it accordingly
	-- state: <string> "ON" or "OFF"
	function search_min:ToggleActive(state)
		local active_color = "255 255 255"
		local inactive_color = "192 192 192"
		if (state) then
			search_min.active = state
			search_min.fgcolor = state == "NO" and inactive_color or active_color
		else
			search_min.active = filter_radio.value.title == "All" and "NO" or "YES"
			search_min.fgcolor = filter_radio.value.title == "All" and inactive_color or active_color
		end
	end
	
	function hamburger:action()
		ShowDialog(BuildHamburgerUI(self, {setpathbtn, clearpathbtn, cursectorbtn, optionsbtn, closebtn}))
	end

	function nextbtn:action()
		change_page(ui_page + 1)
	end

	function prevbtn:action()
		change_page(ui_page - 1)
	end
	
	function cursectorbtn:action()
		T.UI.navmap:SetSectorID(GetCurrentSectorid())
		T.UI.navmap:SetSystemID(GetCurrentSystemid())
		change_page(3)
	end

	function setpathbtn:action()
		NavRoute.SetFinalDestination(navmap:GetSectorID())
		navmap:setpath(GetFullPath(GetCurrentSectorid(), NavRoute.GetCurrentRoute()))
		clearpathbtn.active = "YES"
	end
	
	function clearpathbtn:action()
		NavRoute.clear()
		navmap:setpath(GetFullPath(GetCurrentSectorid(), NavRoute.GetCurrentRoute()))
		self.active = "NO"
	end
	
	function searchbtn:action()
		--if (filter_radio.value.title ~= "All") then
			-- open and populate search matrix
			if (ui_page == 3) then prevbtn:action() end
			ShowDialog(BuildSearchUI(ore_reference[3][filter_radio.value.title]))
		--end
	end
	
	function optionsbtn:action()
		ShowDialog(T.UI.optionsdlg)
	end

	function closebtn:action()
		HideDialog(iup.GetDialog(self))
	end
	
	function reloadbtn:action()
		ReloadInterface()
	end
	
	function targetbtn:action()
		matrix:edition_cb()
	end

	function dlg:SetDescriptionMsg(s)
		T.UI.desc_header.title = s or ""
	end

	function dlg:map_cb()
		-- TODO can this be moved to the matrix definition?
		-- Fix the width of the filler column on the matrix
		-- For some reason NUMCOL is incremented by 1 after mapping
		local total_width = 0
		for i = 1, (matrix.numcol - 2) do -- account for the increased NUMCOL and the filler
			total_width = total_width + matrix["WIDTH"..i]
		end
		matrix["WIDTH"..(matrix.numcol-1)] = getwidth(matrix) - total_width - Font.Default -- Font.Default defines the scrollbar width
	end

	function dlg:show_cb()
		if (IsConnected()) then
			clearpathbtn.active = #NavRoute.GetCurrentRoute() > 0 and "YES" or "NO"
			T.SaveData(GetCurrentSystemid()) -- Attempt to save data.  GetCurrentSystemid() works here because they're opening the GUI in the system.  No possible WH transition.
		end

		if (not ui_page) then -- First run setup
			if (IsConnected()) then
				-- Set the Navmap system and sector IDS on First run, depending on the start_page
				if (start_page > 1) then
					navmap:SetSystemID(GetSystemID(GetCurrentSectorid()))
				end
				if (start_page > 2) then
					navmap:SetSectorID(GetCurrentSectorid())
				end
			end
			change_page(start_page) 
			
		else
			-- Always open to the start_page instead of the page being viewed when the UI is closed
			ui_page = nil
			change_page(start_page)
		end
	end

	-- ---------------------------------------------

		-- Change the UI Page and adjust all relevant controls
	change_page = function(page)
		-- Check if there is sufficient information to change the page.  Default up a page if not safe.
		local systemid = ui_page and navmap:GetSystemID() or page >= 2 and GetCurrentSystemid() -- needed so that the UI can open to system view if that's what the start_page is
		local sectorid = ui_page and navmap:GetSectorID() or page == 3 and GetCurrentSectorid() -- needed so that the UI can open to sector view if that's what the start_page is
		if (page == 3 and not sectorid) then page = 2 end
		if (page == 2 and not systemid) then page = 1 end

		ui_page = page
		navmap:SetMapMode(page)
		T.UI.scanned_total.title = "-"
		T.UI.est_max_total.title = "-"

		-- page is the new page being switched to after clicking Prev or Next
		if (page == 1) then
			dlg:SetDescriptionMsg("MDI RoidDB".." v"..roiddb.version)
			main_display_zbox.value = navmap_container
			navmap:SetSectorID()
			navmap:reset() -- change the visible map

			nextbtn.active = systemid and "YES" or "NO"
			nextbtn.title = "System Map >"
			prevbtn.active = "NO"
			prevbtn.title = ""
			searchbtn.active = "YES" -- filter_radio.value.title == "All" and "NO" or "YES"
			search_min:ToggleActive("YES")
			setpathbtn.active = "NO"

		elseif (page == 2) then
			dlg:SetDescriptionMsg(SystemNames[systemid].." System")
			main_display_zbox.value = navmap_container
			navmap:reset(ore_reference[3][filter_radio.value.title]) -- change the visible map

			nextbtn.active = sectorid and "YES" or "NO"
			nextbtn.title = "Sector List >"
			prevbtn.active = "YES"
			prevbtn.title = "< Universe Map"
			searchbtn.active = "YES" --filter_radio.value.title == "All" and "NO" or "YES"
			search_min:ToggleActive("YES")
			setpathbtn.active = IsConnected() and sectorid and "YES" or "NO"

		elseif (page == 3) then
			dlg:SetDescriptionMsg(LocationStr(sectorid))
			main_display_zbox.value = matrix_container
			local data = T.GetData(systemid, sectorid)--, nil, true) -- don't create a table if it doesn't exist.  keep the data table clean
			matrix:reset(data, ore_reference[3][filter_radio.value.title])
			filters_and_highest:SetHighest(systemid, sectorid)

			targetbtn.active = "NO"--navmap:GetSectorID() == GetCurrentSectorid() and "YES" or "NO"
			navmap:SetInfo("")
			nextbtn.active = "NO"
			nextbtn.title = ""
			prevbtn.active = "YES"
			prevbtn.title = "< System Map"
			searchbtn.active = "NO"
			search_min:ToggleActive("NO")
			setpathbtn.active = IsConnected() and "YES" or "NO"
		end

		if (IsConnected() and #(NavRoute.GetCurrentRoute()) > 0) then
			clearpathbtn.active = "YES"
		end
	end
	-- ---------------------------------------------

	iup.Map(dlg)

	T.UI.maindlg = dlg
	T.UI.matrix = matrix
	T.UI.navmap = navmap
	T.UI.unknownbtn = unknownbtn
	T.UI.nextbtn = nextbtn
	T.UI.prevbtn = prevbtn
	T.UI.targetbtn = targetbtn
	T.UI.setpathbtn = setpathbtn
	T.UI.search_min = search_min
	T.UI.scanned_total = scanned_total
	T.UI.est_max_total = est_max_total
	T.UI.filters_and_highest = filters_and_highest
	T.UI.info_display = info_display
	T.UI.desc_header = desc_header
	
end
T.UI.BuildUI = BuildUI

-- ### ######################################################### ###
-- ### ######################################################### ###
-- ###                                                           ###
-- ###                         Search UI                         ###
-- ###                                                           ###
-- ### ######################################################### ###
-- ### ######################################################### ###

BuildSearchUI = function(filter)
	local matrix_container, matrix
	local desc_header
	local backbtn, closebtn, drilldownbtn, setpathbtn
	local container, dlg

-- ### ######################################################### ###
-- ### Search UI - Interface Layout                              ###
-- ### ######################################################### ###
	
	local layout = ui_page == 1 and T.MatrixLayout["universe_ore_search"] or (ui_page == 2 or ui_page == 3) and T.MatrixLayout["system_ore_search"]
	desc_header   = iup.label{title = "", expand = "HORIZONTAL", font = Font.H1 * T.FONT_SCALE}
	matrix_container, matrix = T:BuildMatrix(layout, {scrollbar = "VERTICAL", size = "xTWOTHIRD"})
	backbtn = iup.stationbutton{title = "Back", active = "NO", font = T.Font}
	closebtn = iup.stationbutton{title = "Close", font = T.Font}
	setpathbtn = iup.stationbutton{title = "Set Path", active = "NO", font = T.Font}
	drilldownbtn = iup.stationbutton{title = "Drill Down", active = "NO", font = T.Font}

	--iup.Detach(matrix)
	container =iup.vbox{
		iup.pdasubsubsubframebg{
			iup.pdarootframe{
				desc_header,
			},
		},
		iup.pdasubsubsubframebg{
			matrix_container,
		},
		iup.pdasubsubsubframebg{
			iup.pdarootframe{
				iup.hbox{
					closebtn,
					iup.fill{},
					backbtn,
					iup.fill{},
					drilldownbtn,
					iup.fill{},
					setpathbtn,
				},
			},
		},
		gap = 10,
		expand = "NO",
	}


	dlg = iup.dialog{
		iup.vbox{
			iup.fill{},
			iup.hbox{
				iup.fill{},
				container,
				iup.fill{},
			},
			iup.fill{},
		},
		-- -------------------
		topmost = "YES",
		fullscreen = "YES",
		bgcolor = "0 0 0 128 *",
		defaultesc = closebtn,
	}
	
-- ### ######################################################### ###
-- ### Search UI - Interface Callbacks & Supporting Code         ###
-- ### ######################################################### ###

	local cur_index

	function closebtn:action()
		HideDialog(iup.GetDialog(self))
	end

	function backbtn:action()
		T.UI.prevbtn:action()
		matrix:reset(T.GetUniverseSummary(), filter, T.MatrixLayout["universe_ore_search"])
		self.active = "NO"
		drilldownbtn.active = "NO"
		setpathbtn.active = "NO"
		dlg:SetHeaderMsg(T.UI.navmap:GetSystemID()) -- Set the header description last so the ui_page has a chance to change

	end
	
	function drilldownbtn:action()
		local sel_desc = matrix:GetCellInCurLine(1)
		local systemid = GetSystemIDFromName(sel_desc)

		-- From the universe summary, prepare and switch to the system summary
		if (ui_page == 1) then
			T.UI.navmap:SetSystemID(systemid)
			T.UI.nextbtn:action() -- change the page and adjust controls accordingly
			backbtn.active =  "YES"
			drilldownbtn.active = "NO"
			-- load the system summary into the matrix
			matrix:reset(T.GetSystemSummary(systemid), filter, T.MatrixLayout["system_ore_search"])
		
		-- From the system summary, prepare and switch to the sector view
		elseif (ui_page == 2) then
			local sectorid = GetSectorIDFromShortStr(sel_desc)
			T.UI.navmap:SetSectorID(sectorid)
			T.UI.nextbtn:action()
			closebtn:action()

-- TODO get search from sector view to work			
		elseif (ui_page == 3) then
			T.UI.navmap:SetSystemID(systemid)
			T.UI.prevbtn:action() -- change the page and adjust controls accordingly
			backbtn.active =  "YES"
			drilldownbtn.active = "NO"
			-- load the system summary into the matrix
			matrix:reset(T.GetSystemSummary(systemid), filter, T.MatrixLayout["system_ore_search"])
		end

		dlg:SetHeaderMsg(systemid) -- Set the header description last so the ui_page has a chance to change

	end
	
	function setpathbtn:action()
		NavRoute.SetFinalDestination(GetSectorIDFromShortStr(matrix:GetCellInCurLine(1)))
		T.UI.navmap:setpath(GetFullPath(GetCurrentSectorid(), NavRoute.GetCurrentRoute()))
		closebtn:action()
	end

	function dlg:SetHeaderMsg(systemid)
		-- header message will always be one of two things dependant on the current page.
		local msg = (ore_reference[2][filter] or "All Ore").." Search"
		if (ui_page == 1) then
			msg = msg.." - Universe"
		else
			msg = msg.." - "..SystemNames[systemid].." System"
		end
		desc_header.title = msg
	end

	function dlg:show_cb()
		-- Adjust the matrix description label to indicate what ore is being searched for.
		local systemid = T.UI.navmap:GetSystemID()
		local data = ui_page == 1 and T.GetUniverseSummary() or T.GetSystemSummary(systemid)
		self:SetHeaderMsg(systemid)
		backbtn.active = ui_page > 1 and "YES" or "NO"
		matrix:reset(data, filter)
	end

	function dlg:close_cb()
		iup.Destroy(dlg)
	end
	
	iup.Map(dlg)

	T.UI.search_backbtn = backbtn
	T.UI.search_setpathbtn = setpathbtn
	T.UI.drilldownbtn = drilldownbtn
	
	return dlg
end


-- ### ######################################################### ###
-- ### ######################################################### ###
-- ###                                                           ###
-- ###                       Hamburger                           ###
-- ###                                                           ###
-- ### ######################################################### ###
-- ### ######################################################### ###
BuildHamburgerUI = function(icon, buttons)
	local dlg, container, cbox
	local ingredients = {}


-- ### ######################################################### ###
-- ### Hamburger - Interface Layout                              ###
-- ### ######################################################### ###

	local w, h = get_control_width("stationbutton", "0")
	for i, c in ipairs(buttons) do
		ingredients[i] = iup.zbox{
			iup.canvas{size = c.size.."x"..(h * 1.25), border = "NO"},
			iup.label{title = c.title, size = c.size, fgcolor = c.active == "NO" and "128 128 128" or "255 255 255"},
			all = "YES",
			alignment = "ALEFT",
			expand = "NO",
		}
	end

	ingredients = iup.vbox(ingredients)
	ingredients.alignment = "ARIGHT"
	container = iup.pdasubsubsubframebg{
		iup.pdarootframe{
			iup.vbox{
				ingredients,
				margin = "3x",
			},
		},
	}

	cbox = iup.cbox{size = "FULLxFULL"}
	iup.Append(iup.GetNextChild(cbox), container)

	dlg = iup.dialog{
		cbox,
		topmost = "YES",
		fullscreen = "YES",
		bgcolor = "0 0 0 0 *",
	}

-- ### ######################################################### ###
-- ### Hamburger - Interface Callbacks & Supporting Code         ###
-- ### ######################################################### ###

	for i, c in ipairs(buttons) do
		ingredients[i][1].button_cb = function(self, but, pressed)
		if (pressed == 0 and c.active == "YES") then
				HideDialog(iup.GetDialog(self))
				c:action()
			end
		end
	end

	function cbox:button_cb()
		HideDialog(iup.GetDialog(self))
	end

	function dlg:show_cb()
		container.cx = icon.x - (container.w - icon.w)
		container.cy = icon.y + icon.h
		cbox.REFRESH = "YES"
	end

	function dlg:close_cb()
		iup.Destroy(self)
	end

	iup.Map(dlg)

	return dlg
end






-- ### ######################################################### ###
-- ### ######################################################### ###
-- ###                                                           ###
-- ###                    Status Indicator                       ###
-- ###                                                           ###
-- ### ######################################################### ###
-- ### ######################################################### ###

BuildStatus = function()

	local dlg, container, hbox
	local frames = {}
-- ### ######################################################### ###
-- ### Status Indicator - Interface Layout                       ###
-- ### ######################################################### ###


	local frame1 = iup.stationhighopacitysubframe{iup.label{title = ""}, bgcolor = "0 255 0 255", size = "50x50", visible = "NO"}
	local frame2 = iup.stationhighopacitysubframe{iup.label{title = ""}, bgcolor = "0 255 0 255", size = "50x50", visible = "NO"}
	local frame3 = iup.stationhighopacitysubframe{iup.label{title = ""}, bgcolor = "0 255 0 255", size = "50x50", visible = "NO"}
	local frame4 = iup.stationhighopacitysubframe{iup.label{title = ""}, bgcolor = "0 255 0 255", size = "50x50", visible = "NO"}
	local frame5 = iup.stationhighopacitysubframe{iup.label{title = ""}, bgcolor = "0 255 0 255", size = "50x50", visible = "NO"}

	hbox = iup.hbox{frame1, frame2, frame3, frame4, frame5, gap = 10}
	roiddb.hbox = hbox
	container = iup.pdarootframe{hbox}
	
	dlg = iup.dialog{
		iup.vbox{
			iup.fill{},
			iup.hbox{
				iup.fill{},
				container,
				iup.fill{},
			},
			iup.fill{},
		},
		fullscreen = "YES",
		topmost = "YES",
		bgcolor = "0 0 0 64 *",
	}

	local num = 1
	function dlg:show_cb()
		local timer = Timer()
		local function loop(down)
			if (hbox[num]) then 
				hbox[num].visible = "YES"
			end
			
			if (down) then
				if (num < 3 and hbox[num + 3]) then hbox[num + 3].visible = "NO" end
				num = num - 1
			else
				if (num > 3 and hbox[num - 3]) then hbox[num - 3].visible = "NO" end
				num = num + 1
				if (num == 9) then
					num = 5
					down = true
				end
			end
			
			if (num ~= -3) then	   
				timer:SetTimeout(1000, function() loop(down) end)
			end
		end
			
		loop()
	end
	
	iup.Map(dlg)
	return dlg
end
--roiddb.status = BuildStatus()


BuildUI()
